@@ -100,6 +100,7 @@ class NotificationProtocol(BaseProtocol, Timer):
         self.__state = ProtocolState.CLOSED
         self._protocol_version = version
         self._callbacks = {} # tr_id=>(callback, errback)
+        self._time_id = 0
 
     # Properties ------------------------------------------------------------
     def __get_state(self):
@@ -155,7 +156,7 @@ class NotificationProtocol(BaseProtocol, Timer):
             @type personal_message: string"""
         cm = ''
         if current_media is not None:
-            cm ='\\0Music\\01\\0{0} - {1}\\0%s\\0%s\\0\\0' % \
+            cm = '\\0Music\\01\\0{0} - {1}\\0%s\\0%s\\0\\0' % \
                 (xml_utils.escape(current_media[0]),
                  xml_utils.escape(current_media[1]))
 
@@ -335,7 +336,7 @@ class NotificationProtocol(BaseProtocol, Timer):
                 host = command.arguments[1]
                 port = self._transport.server[1]
             logger.debug("<-> Redirecting to " + command.arguments[1])
-            self._transport.reset_connection((host,port))
+            self._transport.reset_connection((host, port))
         else: # connect to a switchboard
             try:
                 host, port = command.arguments[1].split(":", 1)
@@ -396,6 +397,8 @@ class NotificationProtocol(BaseProtocol, Timer):
     def _handle_QNG(self, command):
         timeout = int(command.arguments[0])
         self.start_timeout("ping", timeout)
+        self._time_id = time.time()
+        self.start_timeout(("qing", self._time_id), timeout+5)
 
     def _handle_OUT(self, command):
         reason = None
@@ -552,14 +555,17 @@ class NotificationProtocol(BaseProtocol, Timer):
 
     # --------- Contact List -------------------------------------------------
     def _handle_ADL(self, command):
-        if command.transaction_id == 0: # incoming ADL from the server
-            self._client.address_book.check_pending_invitations()
         if len(command.arguments) > 0 and command.arguments[0] == "OK":
-            if self._state != ProtocolState.OPEN: # Initial ADL
+            # Confirmation for one of our ADLs
+            if command.transaction_id != 0 \
+            and self._state != ProtocolState.OPEN:
+                # Initial ADL
                 self._state = ProtocolState.OPEN
                 self._transport.enable_ping()
-            else: # contact Added
-                pass
+        else:
+            if command.payload:
+                # Incoming payload ADL from the server
+                self._client.address_book.sync(True)
 
     def _handle_RML(self, command):
         pass
@@ -684,7 +690,7 @@ class NotificationProtocol(BaseProtocol, Timer):
     # --------- Invitation ---------------------------------------------------
     def _handle_RNG(self, command):
         session_id = command.arguments[0]
-        host, port = command.arguments[1].split(':',1)
+        host, port = command.arguments[1].split(':', 1)
         port = int(port)
         key = command.arguments[3]
         account = command.arguments[4]
@@ -695,9 +701,6 @@ class NotificationProtocol(BaseProtocol, Timer):
         self.emit("switchboard-invitation-received", session, inviter)
 
     # --------- Challenge ----------------------------------------------------
-    def _handle_QNG(self, command):
-        pass
-
     def _handle_QRY(self, command):
         pass
 
@@ -708,7 +711,31 @@ class NotificationProtocol(BaseProtocol, Timer):
 
     # --------- Notification -------------------------------------------------
     def _handle_NOT(self, command):
-        pass
+        notification_xml = xml_utils.unescape(command.payload)
+        notification = ElementTree.fromstring(notification_xml)
+
+        service = notification.findtext('MSG/BODY/NotificationData/Service')
+        if service != 'ABCHInternal':
+            return
+
+        try:
+            notification_id = notification.attrib['id']
+            site_id = notification.attrib['siteid']
+            message_id = notification.find('MSG').attrib['id']
+            send_device = notification.find('TO/VIA').attrib['agent']
+            receiver_cid = notification.findtext('MSG/BODY/NotificationData/CID')
+            receiver_account = notification.find('TO').attrib['name'].lower()
+
+            if notification_id != '0' or site_id != '45705' \
+            or message_id != '0' or send_device != 'messenger' \
+            or receiver_cid != str(self._client.profile.cid)  \
+            or receiver_account != self._client.profile.account.lower():
+                return
+
+        except KeyError:
+            return
+
+        self._client.address_book.sync(True)
 
     #---------- Errors -------------------------------------------------------
     def _error_handler(self, error):
@@ -748,6 +775,10 @@ class NotificationProtocol(BaseProtocol, Timer):
     def on_ping_timeout(self):
         self._transport.enable_ping()
 
+    def on_qing_timeout(self, time_id):
+        if self._time_id == time_id:
+            self._transport.emit("connection-lost", "Ping timeout")
+
     def _command_answered_cb(self, tr_id, *args):
         callback, errback = self._callbacks.get(tr_id, (None, None))
         run(callback, *args)
@@ -816,7 +847,7 @@ class NotificationProtocol(BaseProtocol, Timer):
 
         if contact.network_id != profile.NetworkID.MOBILE:
             account, domain = contact.account.split('@', 1)
-            payload = '<ml l="2"><d n="%s"><c n="%s"/></d></ml>'% \
+            payload = '<ml l="2"><d n="%s"><c n="%s"/></d></ml>' % \
                     (domain, account)
             self._send_command("FQY", payload=payload)
 
